# Download and install these packages:
suppressMessages(library(dplyr))
library(unmarked)

# This script uses that provided by Fiske & Chandler 2011 as part of their
# Occupancy modeling examples as a template. Original available at:
# https://sites.google.com/site/unmarkedinfo/home 

# Fiske, I. and R.B. Chandler.
# 2011. unmarked: An R package for fitting hierarchical models of wildlife
# occurrence and abundance. Journal of Statistical Software, 43(10), 1-23. URL 
# http://www.jstatsoft.org/v43/i10/.

# -------------------------- PART I. Format data -------------------------
# This is occupancy modelling for a 2012 point count survey across the biome. I
# use summer and winter as repeats subset the data we might need:

rawdata <- read.csv("fynbosbirds2012.csv")

data <- select(rawdata, ID2, Survey, arrive, Month, VegHt, S, E, START, time, roberts, SPECIES, distfordist, n, altitude, aspect, fire, habitat.1, habitat, wind, temp)

# environment and data cleaning:
rm(rawdata)

data$Month <- as.factor(data$Month)
data$ID2 <- as.factor(data$ID2)
data$roberts <- as.factor(data$roberts)
data$aspect[data$aspect=="s"] <- c("S")
data$aspect[data$aspect=="sw"] <- c("SW")
data$aspect <- factor(data$aspect)
table(data$habitat.1)
data$habitat.1[data$habitat.1=="Asteraceous"] <- c("Dry Scrub")
data$habitat.1[data$habitat.1=="Forest"] <- c("Thicket")
#rename habitat.1
names(data)
names(data)[17] <- c("habitat")
# this selects encounters <50m from the observer so birds are related to enviro variables
names(data)[18] <- c("hab_selector") 
data$habitat <- factor(data$habitat)

# Means group size to fill in blanks for group size where detection was aural only
temp <- subset(data, data$hab_selector==1)
by_spp <- group_by(temp, roberts)
group_means <- summarise(by_spp, means=mean(na.omit(n)))
rm(temp)

#replace n NAs with mean group size

for(i in 1:nrow(data)){if(is.na(data$n[i])){
        x <- as.numeric(as.character(data$roberts[i]))
        if(x %in% group_means$r2){
        y <- as.numeric(group_means[group_means$r2==x,][2])
        data$n2[i] <- y}
}
else (data$n2[i] <- data$n[i] )
}

## make a matrix with numbers of birds per survey

# first select surveyed points per survey
surveyedinsummer <- data %>% select(ID2, Survey,START) %>% filter(Survey=="summer"&!is.na(START))
surveyedinwinter <- data %>% select(ID2, Survey,START) %>% filter(Survey=="winter"&!is.na(START))

# get site covariates:
covariates <- data %>% select(Survey,ID2, VegHt, altitude, aspect, fire, habitat, START)%>%
        filter(Survey=="summer"&!is.na(START))
covariates <- select(covariates, -Survey, -START)
# convert fire to category factor based on quantiles
quantile(na.omit(covariates$fire))
firecat <- rep(NA, nrow(covariates))
for(i in 1:nrow(covariates)){
        if(is.na(covariates$fire[i])){firecat[i] <- c("n.a.")}else{
                if(covariates$fire[i]<40) {firecat[i] <- c(">13")}                        
                if(covariates$fire[i]<=13.1){ firecat[i] <- c("<13")}
                if(covariates$fire[i]<=9.1){ firecat[i] <- c("<9")}
                if(covariates$fire[i]<=4.1){firecat[i] <- c("<4")}}
}
covariates <- cbind(covariates, firecat)
covariates$firecat <- factor(covariates$firecat)
# Standardize site-covariates: 
altitude.mean <- mean(na.omit(covariates$altitude))
altitude.sd <- sd(na.omit(covariates$altitude))
altitude.z <- (covariates$altitude-altitude.mean)/altitude.sd
covariates$altitude <- altitude.z

VegHt.mean <- mean(covariates$VegHt)
VegHt.sd <- sd(covariates$VegHt)
VegHt.z <- (covariates$VegHt-VegHt.mean)/VegHt.sd
covariates$VegHt <- VegHt.z

# # get observation covariates:
obswinter <- data %>% select(Survey, ID2, START, arrive, temp)%>%filter(Survey=="winter"&!is.na(START))%>%select(ID2, temp, arrive)
names(obswinter)[2] <- c("wintertemp")
names(obswinter)[3] <- c("wintertime")

obssummer <- data %>% select(Survey, ID2, START, arrive, temp)%>%filter(Survey=="summer"&!is.na(START))%>%select(ID2, temp, arrive)
names(obssummer)[2] <- c("summertemp")
names(obssummer)[3] <- c("summertime")

obscov <- left_join(obssummer, obswinter) # you use this for any species e.g. below


# Function: species selector: For this function you can select any species in
# the data set using its ADU (roberts) number
# e.g. sppselector(data, 749) will select Cape Sugarbird
# selector = T will select only observations within 50m of the observer. Change this to 
# F (FALSE) for species with low number of detections. 

sppselector <- function(df, x, selector=T){
        summer <- df %>% 
                filter(hab_selector==1, roberts==x, !is.na(n2)) %>%
                select (ID2, Survey, n2) %>% 
                filter (Survey=="summer") %>%
                group_by(ID2) %>%
                summarise(nbirds=sum(n2))
        names(summer)[2] <- c("nsummer")
        
        #again for winter
        winter <- df %>% 
                filter(hab_selector==1, roberts==x, !is.na(n2)) %>%
                select (ID2, Survey, n2) %>% 
                filter (Survey=="winter") %>%
                group_by(ID2) %>%
                summarise(nbirds=sum(n2))
        names(winter)[2] <- c("nwinter")
        #join
        temps <- left_join(surveyedinsummer, summer)
        temps<- temps[,-c(2, 3)]
        for(i in 1:nrow(temps)){if(is.na(temps$nsummer[i])) temps$nsummer[i] <- 0} #turn NA into 0
        
        tempw <- left_join(surveyedinwinter, winter)
        tempw <- tempw[,-c(2, 3)]
        for(i in 1:nrow(tempw)){if(is.na(tempw$nwinter[i])) tempw$nwinter[i] <- 0}
        
        return(detections <- left_join(temps, tempw))
}

# Example of occupancy modelling for Cape Sugarbird:

#initialise the data frame of Cape Sugarbird encounters
csb <- sppselector(data, 749) 
suppressWarnings(csb[csb>1]<- 1) #convert encounters to binary

# Build the unmarked frame (umf):
csb.umf <- unmarkedFrameOccu(y=csb[,2:3],
                             siteCovs=covariates[,2:7],
                             obsCovs=list(temps=obscov[,c(2,4)], 
                                          time=obscov[,c(3,5)]))
summary(csb.umf)

# standardize covariates after making the unmarked frame 
obsCovs(csb.umf) <- scale(obsCovs(csb.umf))
summary(csb.umf)

# default model:
fm1 <- occu(~1 ~1, csb.umf)
backTransform(fm1, type="state")#19% occupancy of sites
backTransform(fm1, type="det")#39% probability of detection

#first see best variables associated with detection probability
# by creating various alternative models:
fm2 <- occu(~temps ~1, csb.umf)
fm3 <- occu(~time ~1, csb.umf)
fm4 <- occu(~VegHt ~1, csb.umf)
fm5 <- occu(~temps*time ~1, csb.umf)
fm6 <- occu(~temps*time+VegHt ~1, csb.umf)

# PART III. Model selection, prediction, model-averaging ------

# Model selection via AIC

# first put the fitted models in a "fitList"
fms <- fitList(
        "psi(.)p(.)"            = fm1,
        "psi(.)p(temps)"        = fm2,
        "psi(.)p(time)"         = fm3,
        "psi(.)p(Veg)"          = fm4,
        "psi(.)p(temp+time)"    =fm5,
        "psi(.)p(temp+time+veg)"=fm6
)

# Rank them by AIC
(ms <- modSel(fms))
# veg is most important for csb ie. fm4. 

#try understand how veg influences detectability
summary(csb.umf)
newData1 <- data.frame(VegHt=seq(-1.7, 2.5, by=0.1), temps=0, time=0)#seq taken from standardization(above)
E.p <- predict(fm4, type="det", newdata=newData1, appendData=TRUE)
head(E.p)

# Plot it
plot(Predicted ~ VegHt, E.p, type="l", ylim=c(0,1),
     xlab="veght (standardized)",
     ylab="Expected detection probability")
lines(lower ~ VegHt, E.p, type="l", col=gray(0.5))
lines(upper ~ VegHt, E.p, type="l", col=gray(0.5))


## now look at occupancy variables with veg as predictor offset:
#recycling model names to save environment space
fm1 <- occu(~VegHt ~altitude, csb.umf)
fm2 <- occu(~VegHt ~VegHt, csb.umf)
fm3 <- occu(~VegHt ~habitat, csb.umf)
fm4 <- occu(~VegHt ~VegHt*altitude, csb.umf)
fm5 <- occu(~VegHt ~VegHt*altitude+habitat, csb.umf)
fm6 <- occu(~VegHt ~firecat, csb.umf)
fm7 <- occu(~VegHt ~VegHt*altitude+firecat, csb.umf)
fm8 <- occu(~VegHt ~VegHt*altitude+firecat+habitat, csb.umf)

fms <- fitList(
        "psi(altitude)p()"           = fm1,
        "psi(veg)p()"               = fm2,
        "psi(habitat)p()"            = fm3,
        "psi(veg*alt)p()"            = fm4,
        "psi(veg*alt+hab)p()"        = fm5,
        "psi(firecat)p()"            = fm6,
        "psi(veg*alt+fire)p()"        = fm7,
        "psi(veg*alt+fire+habitat)p()"= fm8#best model for csb 
)

# Rank them by AIC
(ms <- modSel(fms))

